package Package1;

// La classe Fraction implementa frazioni, ossia numeri razionali (rapporti di numeri interi)
class Fraction {

    private int numerator;
    private int denominator;

    public Fraction() {
        numerator = 0;
        denominator = 1;
    }

    public Fraction(int n, int d) {
        if (d == 0) {
            numerator = 0;
            denominator = 1;
        } else {
            numerator = n;
            denominator = d;
        }
    }

    public Fraction(int n) {
        numerator = n;
        denominator = 1;
    }

    @Override
    public String toString() {
        return (numerator + "/" + denominator);
    }

    public double evaluate() {
        double n = numerator;	// conversione a double (alternativa al cast di uno degli operandi)
        double d = denominator;
        return (n / d);
    }

    public Fraction add(Fraction f2) {
        Fraction r = new Fraction((numerator * f2.denominator)
                + (f2.numerator * denominator),
                (denominator * f2.denominator));
        r.simplify();
        return r;
    }

    public Fraction subtract(Fraction f2) {
        Fraction r = new Fraction((numerator * f2.denominator)
                - (f2.numerator * denominator),
                (denominator * f2.denominator));
        r.simplify();
        return r;
    }

    public Fraction multiply(Fraction f2) {
        Fraction r = new Fraction(numerator * f2.numerator, denominator * f2.denominator);
        r.simplify();
        return r;
    }

    public Fraction multiply(int value) {
        Fraction r = new Fraction(numerator * value, denominator);
        r.simplify();
        return r;
    }

    public Fraction divide(Fraction f2) {
        Fraction r = new Fraction(numerator * f2.denominator, denominator * f2.numerator);
        r.simplify();
        return r;
    }

    public Fraction divide(int value) {
        Fraction r = new Fraction(numerator, denominator * value);
        r.simplify();
        return r;
    }

    // Massimo Comun Divisore: implementazione ricorsiva
    public static int gcd(int p, int q) {
        if (q == 0) {
            return p;
        } else {
            return gcd(q, p % q);
        }
    }

    // Massimo Comun Divisore: implementazione iterativa
    /*
    public static int gcd(int p, int q) {
     	while (q != 0) {
            int temp = q;
            q = p % q;
            p = temp;
     	}
     	return p;
    }
    */

    public void simplify() {
        int thisGcd = gcd(numerator, denominator);
        numerator = numerator / thisGcd;
        denominator = denominator / thisGcd;
    }

    public static void main(String[] argv) {

        // Test dei costruttori e del metodo toString
        Fraction f0 = new Fraction();
        Fraction f1 = new Fraction(3);
        Fraction f2 = new Fraction(12, 20);

        System.out.println("\nTesting constructors (and toString):");
        System.out.println("The fraction f0 is " + f0.toString());
        System.out.println("The fraction f1 is " + f1); // toString è implicito
        System.out.println("The fraction f2 is " + f2);

        // Test del metodo evaluate
        System.out.println("\nTesting add and evaluate:");
        System.out.println("The floating point value of " + f1 + " is "
                + f1.evaluate());
        System.out.println("The floating point value of " + f2 + " is "
                + f2.evaluate());

        // Test dei metodi aritmetici
        System.out.println("\nTesting arithmetic functions:");
        System.out.println("The result of " + f1 + " + " + f2 + " is " + f1.add(f2));
        System.out.println("The result of " + f2 + " - " + f1 + " is " + f2.subtract(f1));
        System.out.println("The result of " + f1 + " x " + f2 + " is " + f1.multiply(f2));
        System.out.println("The result of (" + f2 + ") / 3 is " + f2.divide(3));

        // Test del metodo gcd (massimo comun divisore)
        System.out.println("\nTesting gcd:");
        System.out.println("The gcd of 2 and 10 is: " + gcd(2, 10));
        System.out.println("The gcd of 15 and 5 is: " + gcd(15, 5));
        System.out.println("The gcd of 24 and 18 is: " + gcd(24, 18));
        System.out.println("The gcd of 10 and 10 is: " + gcd(10, 10));
        System.out.println("The gcd of 21 and 400 is: " + gcd(21, 400));
    }
}
