package audio;

public class TestWavetableAlternatives {

    static class GenerateSine {

        static private double[] myWavetable;

        public static void loadWavetable() {
            // generazione di una wavetable (4096 elementi)
            int nSamplesInTable = 4096;
            myWavetable = new double[nSamplesInTable];
            for (int i = 0; i < nSamplesInTable; i++)
                myWavetable[i] = Math.sin(2 * Math.PI * ((double) i / nSamplesInTable));
        }

        public static double[] calculateAll(int nSamples, double frequency, int amplitude) {
            double[] result = new double[nSamples];
            double nSamplesPerCycle = 44100 / frequency;
            for (int i = 0; i < nSamples; i++)
                result[i] = amplitude * Math.sin(2 * Math.PI * (i / nSamplesPerCycle));
            return result;
        }

        public static double[] calculateOneCycle(int nSamples, double frequency, int amplitude) {
            int nSamplesPerCycle = (int) (44100 / frequency); // va effettuato il cast perché nel seguito lo useremo come indice
            double[] table = new double[nSamplesPerCycle];
            for (int i = 0; i < nSamplesPerCycle; i++)
                table[i] = amplitude * Math.sin(2 * Math.PI * ((double) i / nSamplesPerCycle));

            double[] result = new double[nSamples];
            // lettura ciclica della tabella
            for (int k = 0; k < nSamples; k++)
                result[k] = table[k % nSamplesPerCycle];
            return result;
        }

        public static double[] wavetableLookupNoInterpolation(int nSamples, double frequency, int amplitude) {
            int nSamplesPerCycle = (int) (44100 / frequency); // va effettuato il cast perché nel seguito lo useremo come indice
            double[] result = new double[nSamples];
            // lettura ciclica della tabella
            for (int k = 0; k < nSamples; k++) {
                int index = k % nSamplesPerCycle;
                // Attenzione: le operazioni aritmetiche che non coinvolgono la variabile di controllo dovrebbero essere calcolate al di fuori del ciclo
                // Ad esempio (myWavetable.length / nSamplesPerCycle)
                int rescaledIndex = (int) ((double) index * myWavetable.length / nSamplesPerCycle);
                result[k] = myWavetable[rescaledIndex] * amplitude;
            }
            return result;
        }

        public static double[] wavetableLookupInterpolation(int nSamples, double frequency, int amplitude) {
            int nSamplesPerCycle = (int) (44100 / frequency); // va effettuato il cast perché nel seguito lo useremo come indice
            double[] result = new double[nSamples];
            // lettura ciclica della tabella
            for (int k = 0; k < nSamples; k++) {
                int index = k % nSamplesPerCycle;
                double rescaledIndex = ((double) index * myWavetable.length / nSamplesPerCycle);
                int floor = (int) Math.floor(rescaledIndex);
                result[k] = (rescaledIndex - floor) * (myWavetable[floor + 1] - myWavetable[floor]) + myWavetable[floor];
            }
            return result;
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {

        GenerateSine.loadWavetable();

        long startTime = System.currentTimeMillis();
        double[] wave1 = GenerateSine.calculateAll(440000, 220, 1);
        long endTime = System.currentTimeMillis();
        System.out.println("Calcolo completo: " + (endTime - startTime) + " ms");

        startTime = System.currentTimeMillis();
        double[] wave2 = GenerateSine.calculateOneCycle(440000, 220, 1);
        endTime = System.currentTimeMillis();
        System.out.println("Calcolo del ciclo: " + (endTime - startTime) + " ms");

        startTime = System.currentTimeMillis();
        double[] wave3 = GenerateSine.wavetableLookupNoInterpolation(440000, 220, 1);
        endTime = System.currentTimeMillis();
        System.out.println("Wavetable senza interpolazione: " + (endTime - startTime) + " ms");

        startTime = System.currentTimeMillis();
        double[] wave4 = GenerateSine.wavetableLookupInterpolation(440000, 220, 1);
        endTime = System.currentTimeMillis();
        System.out.println("Wavetable con interpolazione lineare: " + (endTime - startTime) + " ms");

        System.out.println();
        System.out.println("Confronto dei risultati (primi 440 valori)");
        for (int i = 0; i < 440; i++)
            System.out.println(wave1[i] + " " + wave2[i] + " " + wave3[i] + " " + wave4[i]);
    }

}
