package jSyn;

import com.jsyn.JSyn;
import com.jsyn.Synthesizer;
import com.jsyn.unitgen.LineOut;
import com.jsyn.unitgen.SineOscillator;
import com.softsynth.shared.time.TimeStamp;

public class PlayNotes {
    Synthesizer synth;
    SineOscillator osc;
    LineOut lineOut;

    private void test() {
        // Create a context for the synthesizer.
        synth = JSyn.createSynthesizer();

        // Add a tone generator.
        synth.add(osc = new SineOscillator() );
        // Add an output mixer.
        synth.add(lineOut = new LineOut());

        // Connect the oscillator to the left and right audio output.
        osc.getOutput().connect(0, lineOut.input, 0);
        osc.getOutput().connect(0, lineOut.input, 1);

        // Start synthesizer using default stereo output at 44100 Hz.
        synth.start();

        // Get synthesizer time in seconds.
        double timeNow = synth.getCurrentTime();

        // Advance to a near future time so we have a clean start.
        TimeStamp timeStamp = new TimeStamp(timeNow + 0.5);

        // We only need to start the LineOut. It will pull data from the oscillator
        lineOut.start(timeStamp);

        // Schedule a note on and off.
        double freq = 220.0; // hertz
        double timeBetweenNotes = 1.0;
        double noteDuration = 0.3;
        osc.noteOn(freq, 0.5, timeStamp);
        osc.noteOff(timeStamp.makeRelative(noteDuration));

        // Schedule this to happen a bit later.
        timeStamp = timeStamp.makeRelative(timeBetweenNotes);
        freq *= 1.5; // up a perfect fifth
        osc.noteOn(freq, 0.5, timeStamp);
        osc.noteOff(timeStamp.makeRelative(noteDuration));

        timeStamp = timeStamp.makeRelative(timeBetweenNotes);
        freq *= 4.0 / 3.0; // up a perfect fourth
        osc.noteOn(freq, 0.5, timeStamp);
        osc.noteOff(timeStamp.makeRelative(noteDuration));

        // Sleep while the song is being generated in the background thread.
        try {
            System.out.println("Sleep while synthesizing.");
            synth.sleepUntil(timeStamp.getTime() + 2.0);
            System.out.println("Woke up...");
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        // Stop everything.
        synth.stop();
    }

    public static void main(String[] args) {
        new PlayNotes().test();
    }
}